package de.mdelab.instanceGraphEditor.ui.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import de.mdelab.instanceGraphEditor.ui.model.Link;
 
/**
 * Command used to create a new bendpoint in a {@linkplain Link}.
 * This class is declared final since it has a very specific functionality.
 *
 */
public final class LinkCreateBendpointCommand extends Command {
 
    /** Index on which the new bendpoint is added. */
    private int index;
    /** Location of new bendpoint. */
    private Point location;
    /** Link to which the bendpoint is added. */
    private Link connection;
 
    @Override public void execute() {
    	connection.addBendpoint(index, location);
    }
 
    @Override public void undo() {
    	connection.removeBendpoint(index);
    }
 
    /**
     * Set the index on which the bendpoint is added.
     * @param index Index on which the bendpoint should be added.
     */
    public void setIndex(final int index) {
        this.index = index;
        //TODO:validation checks.
    }
 
    /**
     * Set the location where the new bendpoint is added.
     * @param location point in the diagram where the new bendpoint
     * is added.
     */
    public void setLocation(final Point location) {
        this.location = location;
    }
 
    /**
     * Set the link on which the new bendpoint is added.
     * @param link link on which the bendpoint is added.
     */
    public void setConnection(final Link connection) {
        this.connection = connection;
    }
}